@@ -139,8 +139,24 @@ void DockRxOpt::setFilterOffset(qint64 freq_hz)
  */
 void DockRxOpt::setFilterOffsetRange(qint64 range_hz)
 {
-    if (range_hz > 0)
-        ui->filterFreq->setup(7, -range_hz/2, range_hz/2, 1, FCTL_UNIT_KHZ);
+    int num_digits;
+
+    if (range_hz <= 0)
+        return;
+
+    range_hz /= 2;
+    if (range_hz < 100e3)
+        num_digits = 5;
+    else if (range_hz < 1e6)
+        num_digits = 6;
+    else if (range_hz < 1e7)
+        num_digits = 7;
+    else if (range_hz < 1e8)
+        num_digits = 8;
+    else
+        num_digits = 9;
+
+    ui->filterFreq->setup(num_digits, -range_hz, range_hz, 1, FCTL_UNIT_KHZ);
 }
 
 /**
@@ -195,7 +211,7 @@ unsigned int DockRxOpt::filterIdxFromLoHi(int lo, int hi) const
  * @param lo Low cutoff frequency in Hz
  * @param hi High cutoff frequency in Hz.
  *
- * This function will automatically select te "User" preset in the
+ * This function will automatically select the "User" preset in the
  * combo box.
  */
 void DockRxOpt::setFilterParam(int lo, int hi)
@@ -287,6 +303,11 @@ void DockRxOpt::setSquelchLevel(double level)
     ui->sqlSpinBox->setValue(level);
 }
 
+double DockRxOpt::getSqlLevel(void) const
+{
+    return ui->sqlSpinBox->value();
+}
+
 /**
  * @brief Get the current squelch level
  * @returns The current squelch setting in dBFS
@@ -326,9 +347,6 @@ void DockRxOpt::readSettings(QSettings *settings)
     int     int_val;
     double  dbl_val;
 
-    bool bool_val = settings->value("gui/fctl_reset_digits", true).toBool();
-    ui->filterFreq->setResetLowerDigits(bool_val);
-
     int_val = settings->value("receiver/cwoffset", 700).toInt(&conv_ok);
     if (conv_ok)
         demodOpt->setCwOffset(int_val);
@@ -486,7 +504,19 @@ void DockRxOpt::setRxFreq(qint64 freq_hz)
 
 void DockRxOpt::setRxFreqRange(qint64 min_hz, qint64 max_hz)
 {
+    ui->freqSpinBox->blockSignals(true);
     ui->freqSpinBox->setRange(1.e-3 * (double)min_hz, 1.e-3 * (double)max_hz);
+    ui->freqSpinBox->blockSignals(false);
+}
+
+void DockRxOpt::setResetLowerDigits(bool enabled)
+{
+    ui->filterFreq->setResetLowerDigits(enabled);
+}
+
+void DockRxOpt::setInvertScrolling(bool enabled)
+{
+    ui->filterFreq->setInvertScrolling(enabled);
 }
 
 /**
@@ -494,7 +524,7 @@ void DockRxOpt::setRxFreqRange(qint64 min_hz, qint64 max_hz)
  * @param freq The new filter offset in Hz
  *
  * This slot is activated when a new filter offset has been selected either
- * usig the mouse or using the keyboard.
+ * using the mouse or using the keyboard.
  */
 void DockRxOpt::on_filterFreq_newFrequency(qint64 freq)
 {
